[{"locale":"es"},{"key":"0061","mappings":{"default":{"default":"a"}},"category":"Ll"},{"key":"0062","mappings":{"default":{"default":"b"}},"category":"Ll"},{"key":"0063","mappings":{"default":{"default":"c"}},"category":"Ll"},{"key":"0064","mappings":{"default":{"default":"d"}},"category":"Ll"},{"key":"0065","mappings":{"default":{"default":"e"}},"category":"Ll"},{"key":"0066","mappings":{"default":{"default":"f"}},"category":"Ll"},{"key":"0067","mappings":{"default":{"default":"g"}},"category":"Ll"},{"key":"0068","mappings":{"default":{"default":"h"}},"category":"Ll"},{"key":"0069","mappings":{"default":{"default":"i"}},"category":"Ll"},{"key":"006A","mappings":{"default":{"default":"j"}},"category":"Ll"},{"key":"006B","mappings":{"default":{"default":"k"}},"category":"Ll"},{"key":"006C","mappings":{"default":{"default":"l"}},"category":"Ll"},{"key":"006D","mappings":{"default":{"default":"m"}},"category":"Ll"},{"key":"006E","mappings":{"default":{"default":"n"}},"category":"Ll"},{"key":"006F","mappings":{"default":{"default":"o"}},"category":"Ll"},{"key":"0070","mappings":{"default":{"default":"p"}},"category":"Ll"},{"key":"0071","mappings":{"default":{"default":"q"}},"category":"Ll"},{"key":"0072","mappings":{"default":{"default":"r"}},"category":"Ll"},{"key":"0073","mappings":{"default":{"default":"s"}},"category":"Ll"},{"key":"0074","mappings":{"default":{"default":"t"}},"category":"Ll"},{"key":"0075","mappings":{"default":{"default":"u"}},"category":"Ll"},{"key":"0076","mappings":{"default":{"default":"v"}},"category":"Ll"},{"key":"0077","mappings":{"default":{"default":"w"}},"category":"Ll"},{"key":"0078","mappings":{"default":{"default":"x"}},"category":"Ll"},{"key":"0079","mappings":{"default":{"default":"y","defaultMP":"ye"}},"category":"Ll"},{"key":"007A","mappings":{"default":{"default":"z"}},"category":"Ll"}]
